# HUC Braille Tables – Readme
Full name: Hexadecimal Unicode Characters Braille Tables

## Table of contents
1. General information
2. Definition for 8-dot (HUC8)
3. Definition for 6-dot (HUC6)
4. Installation instructions for NVDA 2019.2.1 (installed version, UTF-16)
5. Installation instructions for NVDA 2019.2.1 (portable version, UTF-16)
6. Installation instructions for NVDA 2019.3 (installed version, UTF-32)
7. Installation instructions for NVDA 2019.3 (portable version, UTF-32)
8. Update instructions for NVDA 2019.2.1/2019.3 (installed/portable version)
9. Uninstallation instructions for NVDA 2019.2.1/2019.3 (installed/portable version)

## General information
- Inventor/Author/Creator: Daniel Mayr (alias Dr. Sooom)
- Website of the author: https://danielmayr.at/
- Official documentation: https://danielmayr.at/huc/
- Initial release: Friday, March 1, 2019, 16:00 (UTC+1)
- Version: HUCv2
- Release date: Sunday, December 1, 2019
- Changelog file: huc-changelog.md
- License: © 2019 Daniel Mayr (alias Dr. Sooom), LGPL 2.1
- License file: huc-lgpl-2.1.md
- License URL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
- The HUC Braille Tables are compatible with the Liblouis braille tables.
- Known issues are covered in the FAQ section in the official documentation.

## Definition for 8-dot (HUC8)
### Prefix braille characters:
- The prefix character must stand in front of every single Unicode character. To avoid confusion, grouping of two or more unseparated, consecutive Unicode characters, such as ⣥⣺⣩⣥⣺⣩ to ⣥⣺⣩⣺⣩, isn't allowed.
- ⣥ = code points between U+0000 and U+FFFF  
  Code point: U+28E5; Braille dots: 13678  
  Defines the first 65,536 Unicode code points.  
  The prefix character is a combination of the letters u and c.
- ⣭ = code points between U+10000 and U+1FFFF  
  Code point: U+28ED; Braille dots: 134678  
  Defines the second 65,536 Unicode code points.  
  The prefix character is a combination of the letters u and c and the digit 1.  
- ⣽ = code points between U+20000 and U+2FFFF  
  Code point: U+28FD; Braille dots: 1345678  
  Defines the third 65,536 Unicode code points.  
  The prefix character is a combination of the letters u and c and the digit 2.  
- ⣵ = code points between U+30000 and U+10FFFF  
  Code point: U+28F5; Braille dots: 135678  
  Defines the other 917,504 Unicode code points.  
  The prefix character is a combination of the letters u, e and c.  
  And from this point four braille characters (including the prefix braille character) are needed to define a Unicode code point correctly.  
  U+30000 will be changed to U+030000 before converting into braille.  
  In Unicode 12.0 (March 2019) only 337 characters are assigned in these planes.

### List for all 17 Unicode planes:
- ⣥ = code points between U+0000 and U+FFFF
- ⣭ = code points between U+10000 and U+1FFFF
- ⣽ = code points between U+20000 and U+2FFFF
- ⣵⠾ = code points between U+30000 and U+3FFFF
- ⣵⢾ = code points between U+40000 and U+4FFFF
- ⣵⢞ = code points between U+50000 and U+5FFFF
- ⣵⡾ = code points between U+60000 and U+6FFFF
- ⣵⣾ = code points between U+70000 and U+7FFFF
- ⣵⣞ = code points between U+80000 and U+8FFFF
- ⣵⡺ = code points between U+90000 and U+9FFFF
- ⣵⠺ = code points between U+A0000 and U+AFFFF
- ⣵⢺ = code points between U+B0000 and U+BFFFF
- ⣵⣚ = code points between U+C0000 and U+CFFFF
- ⣵⡚ = code points between U+D0000 and U+DFFFF
- ⣵⢚ = code points between U+E0000 and U+EFFFF
- ⣵⠚ = code points between U+F0000 and U+FFFFF
- ⣵⣡ = code points between U+100000 and U+10FFFF

### Converting hexadecimal values into braille:
0 = ⠚, 1 = ⠁, 2 = ⠃, 3 = ⠉, 4 = ⠙, 5 = ⠑, 6 = ⠋, 7 = ⠛  
8 = ⠓, 9 = ⠊, A = ⠈, B = ⠘, C = ⠒, D = ⠂, E = ⠐, F = ⠀

### Combining hexadecimal values:
00 = ⣺, 01 = ⠞, 10 = ⣡, EF = ⠐, FE = ⢀, FF = ⠀

## Definition for 6-dot (HUC6)
### Prefix and suffix braille characters:
- The prefix character must stand in front of every single Unicode character. To avoid confusion, grouping of two or more unseparated, consecutive Unicode characters, such as ⠿⠺⠛⠞⠿⠺⠛⠞ to ⠿⠺⠛⠞⠺⠛⠞, isn't allowed.
- The very last hexadecimal value stands always in the area of the dots 1245 in the fourth braille character. The dots 3 and 6 define the suffix character, which is also always at the fourth position. In other words: The fourth braille character is always a combination of a hexadecimal value and the suffix character. And it's exactly the same way for the fifth braille character for the Unicode code points between U+20000 and U+10FFFF.
- The two non-breaking spaces (" ") between the prefix and the suffix character are only placeholders for three hexadecimal values. The fourth last hexadecimal value is located in the area of the dots 1245 in the first braille character, the third last one is split between the first and the second braille character (dots 36 and 14) and the second last one is located in the area of the dots 2356 in the second braille character. The last two hexadecimal values of a Unicode code point are the most important one. Therefore they shouldn't be split between two 6-dot braille characters.
- And to avoid misunderstanding, ⠀ (U+2800, braille dot 0) isn't allowed as a suffix character.
- ⠿  ⠄ = code points between U+0000 and U+FFFF  
  Code points: U+283F and U+2804; Braille dots: 123456 and 3  
  Defines the first 65,536 Unicode code points.
- ⠿  ⠠ = code points between U+10000 and U+1FFFF  
  Code points: U+283F and U+2820; Braille dots: 123456 and 6  
  Defines the second 65,536 Unicode code points.
- ⠿  ⠤⠇ = code points between U+20000 and U+2FFFF  
  Code points: U+283F, U+2824 and U+2807; Braille dots: 123456, 36 and 123  
  Defines the third 65,536 Unicode code points.  
  And from this point five braille characters (including the prefix braille character) are needed to define a Unicode code point correctly.  
  In Unicode 12.0 (March 2019) 60,859 characters are assigned in the plane U+2xxxx and 337 more in the planes higher than U+30000.  
  The first hexadecimal value for U+2xxxx must stand behind the fourth braille character in the area of the dots 1245 to define the Unicode plane correctly.  
  And for the Unicode code points from U+100000 to U+10FFFF the braille character ⠥ (U+2825, braille dots 136) is used to define this Unicode plane.

### List for all 17 Unicode planes:
- ⠿  ⠄ = code points between U+0000 and U+FFFF
- ⠿  ⠠ = code points between U+10000 and U+1FFFF
- ⠿  ⠤⠇ = code points between U+20000 and U+2FFFF
- ⠿  ⠤⠍ = code points between U+30000 and U+3FFFF
- ⠿  ⠤⠝ = code points between U+40000 and U+4FFFF
- ⠿  ⠤⠕ = code points between U+50000 and U+5FFFF
- ⠿  ⠤⠏ = code points between U+60000 and U+6FFFF
- ⠿  ⠤⠟ = code points between U+70000 and U+7FFFF
- ⠿  ⠤⠗ = code points between U+80000 and U+8FFFF
- ⠿  ⠤⠎ = code points between U+90000 and U+9FFFF
- ⠿  ⠤⠌ = code points between U+A0000 and U+AFFFF
- ⠿  ⠤⠜ = code points between U+B0000 and U+BFFFF
- ⠿  ⠤⠖ = code points between U+C0000 and U+CFFFF
- ⠿  ⠤⠆ = code points between U+D0000 and U+DFFFF
- ⠿  ⠤⠔ = code points between U+E0000 and U+EFFFF
- ⠿  ⠤⠄ = code points between U+F0000 and U+FFFFF
- ⠿  ⠤⠥ = code points between U+100000 and U+10FFFF

### Converting hexadecimal values into braille:
0 = ⠚, 1 = ⠁, 2 = ⠃, 3 = ⠉, 4 = ⠙, 5 = ⠑, 6 = ⠋, 7 = ⠛  
8 = ⠓, 9 = ⠊, A = ⠈, B = ⠘, C = ⠒, D = ⠂, E = ⠐, F = ⠀

### Combining hexadecimal values:
0000 = ⠺⠽⠚, 0001 = ⠺⠽⠁, 0010 = ⠺⠋⠚, 0100 = ⠞⠴⠚  
1000 = ⠡⠽⠚, FFEF = ⠀⠠⠀, FFFE = ⠀⠀⠐, FFFF = ⠀⠀⠀

## Installation instructions for NVDA 2019.2.1 (installed version, UTF-16)
Notes: To perform the following instructions administrator privileges are required. These installation instructions apply to NVDA 2013.2 (and possibly also to earlier versions) up to NVDA 2019.2.1.
1. Download the latest version of the HUC Braille Tables and open them with 7-Zip.
2. As NVDA 2019.2.1 doesn't support UTF-32, only the following files have to be copied from the 7z archive:
   - "huc8-utf16.tbl" and "huc8-u+0000-u+ffff.tbi" or
   - "huc6-utf16.tbl" and "huc6-u+0000-u+ffff.tbi"
3. Select these two files and press F5 to copy them.
4. As NVDA is installed by default in "C:\Program Files (x86)\NVDA\", please choose the Windows Desktop or any other folder, where administrator privileges aren't required for writing, to copy these two files temporary to another folder.
5. Then select these just copied files on your Desktop or in your previous chosen destination folder via the Windows Explorer and move them to the folder "louis\tables\" in the folder, where NVDA is installed.  
   Note: If NVDA is installed in "C:\Program Files (x86)\NVDA\", you need administrator privileges to proceed this operation.
6. After you have found the name of the braille table, which you want to extend with the HUC8 or with the HUC6 Braille Tables, open that file with Notepad++ (or with any other source code editor) and insert "include huc8-utf16.tbl" or "include huc6-utf16.tbl" followed by a line break at the end of the file.  
   Notes: The very last line must be blank, otherwise the braille output fails. And depending on the NVDA program folder you need administrator privileges here again.
7. Save the file, restart NVDA and the HUC Braille Tables should work. If not, then please take a look at the FAQ section in the official documentation.

## Installation instructions for NVDA 2019.2.1 (portable version, UTF-16)
Notes: To perform the following instructions administrator privileges normally shouldn't be required. These installation instructions apply to NVDA 2013.2 (and possibly also to earlier versions) up to NVDA 2019.2.1.
1. Download the latest version of the HUC Braille Tables and open them with 7-Zip.
2. As NVDA 2019.2.1 doesn't support UTF-32, only the following files have to be copied from the 7z archive:
   - "huc8-utf16.tbl" and "huc8-u+0000-u+ffff.tbi" or
   - "huc6-utf16.tbl" and "huc6-u+0000-u+ffff.tbi"
3. Select these two files and press F5 to copy them.
4. As NVDA is installed as a portable application, you only have to choose the folder "louis\tables\" in the NVDA program folder as the destination folder.
5. After you have found the name of the braille table, which you want to extend with the HUC8 or with the HUC6 Braille Tables, open that file with Notepad++ (or with any other source code editor) and insert "include huc8-utf16.tbl" or "include huc6-utf16.tbl" followed by a line break at the end of the file.  
   Note: The very last line must be blank, otherwise the braille output fails.
6. Save the file, restart NVDA and the HUC Braille Tables should work. If not, then please take a look at the FAQ section in the official documentation.

## Installation instructions for NVDA 2019.3 (installed version, UTF-32)
Notes: To perform the following instructions administrator privileges are required. These installation instructions apply to NVDA 2019.3 and later.
1. Download the latest version of the HUC Braille Tables and open them with 7-Zip.
2. Select all files and press F5 to copy them.
3. As NVDA is installed by default in "C:\Program Files (x86)\NVDA\", please choose the Windows Desktop or any other folder, where administrator privileges aren't required for writing, to copy these two files temporary to another folder.
4. Then select these just copied files on your Desktop or in your previous chosen destination folder via the Windows Explorer and move them to the folder "louis\tables\" in the folder, where NVDA is installed.  
   Note: If NVDA is installed in "C:\Program Files (x86)\NVDA\", you need administrator privileges to proceed this operation.
5. After you have found the name of the braille table, which you want to extend with the HUC8 or with the HUC6 Braille Tables, open that file with Notepad++ (or with any other source code editor) and insert "include huc8-utf16.tbl" or "include huc6-utf16.tbl" followed by a line break at the end of the file.  
   Notes: The very last line must be blank, otherwise the braille output fails. And depending on the NVDA program folder you need administrator privileges here again.
6. To reduce the loading/compiling time of all used braille table files on NVDA startup and after changing the braille output/input table in the NVDA settings, only the first two Unicode planes are included in "huc8-utf32.tbl" and in "huc6-utf32.tbl" by default. If you want that all Unicode code points from U+20000 to U+2FFFF are also displayed in HUC8 respectively in HUC6 style, then please open the corresponding tbl file with Notepad++ (or with any other source code editor) and delete the hash ("#") directly in front of the "include" in line 40.  
   Notes: HUCv2 already contains tbi files for all 17 Unicode planes. And the more of these tbi files are included in a tbl file, the longer NVDA requires to load them all.
7. Save all files, restart NVDA and the HUC Braille Tables should work. If not, then please take a look at the FAQ section in the official documentation.

## Installation instructions for NVDA 2019.3 (portable version, UTF-32)
Notes: To perform the following instructions administrator privileges normally shouldn't be required. These installation instructions apply to NVDA 2019.3 and later.
1. Download the latest version of the HUC Braille Tables and open them with 7-Zip.
2. Select all files and press F5 to copy them.
3. As NVDA is installed as a portable application, you only have to choose the folder "louis\tables\" in the NVDA program folder as the destination folder.
4. After you have found the name of the braille table, which you want to extend with the HUC8 or with the HUC6 Braille Tables, open that file with Notepad++ (or with any other source code editor) and insert "include huc8-utf16.tbl" or "include huc6-utf16.tbl" followed by a line break at the end of the file.  
   Note: The very last line must be blank, otherwise the braille output fails.
5. To reduce the loading/compiling time of all used braille table files on NVDA startup and after changing the braille output/input table in the NVDA settings, only the first two Unicode planes are included in "huc8-utf32.tbl" and in "huc6-utf32.tbl" by default. If you want that all Unicode code points from U+20000 to U+2FFFF are also displayed in HUC8 respectively in HUC6 style, then please open the corresponding tbl file with Notepad++ (or with any other source code editor) and delete the hash ("#") directly in front of the "include" in line 40.  
   Notes: HUCv2 already contains tbi files for all 17 Unicode planes. And the more of these tbi files are included in a tbl file, the longer NVDA requires to load them all.
6. Save all files, restart NVDA and the HUC Braille Tables should work. If not, then please take a look at the FAQ section in the official documentation.

## Update instructions for NVDA 2019.2.1/2019.3 (installed/portable version)
Notes: To perform the following instructions administrator privileges are required depending where and how NVDA is installed. These update instructions apply to NVDA 2013.2 (and possibly also to earlier versions) up to NVDA 2019.2.1 as well as to NVDA 2019.3 and later.
1. Follow the installation instructions for:
   - NVDA 2019.2.1 (installed version, UTF-16)
   - NVDA 2019.2.1 (portable version, UTF-16)
   - NVDA 2019.3 (installed version, UTF-32)
   - NVDA 2019.3 (portable version, UTF-32)
2. During the copying process overwrite all existing files starting with "huc".
3. Skip the step that describes how a braille table is extended with the HUC8 or the HUC6 Braille Tables.
4. Restart NVDA and the newer HUC Braille Tables version should work.

## Uninstallation instructions for NVDA 2019.2.1/2019.3 (installed/portable version)
Notes: To perform the following instructions administrator privileges are required depending where and how NVDA is installed. These uninstallation instructions apply to NVDA 2013.2 (and possibly also to earlier versions) up to NVDA 2019.2.1 as well as to NVDA 2019.3 and later.
1. After you have found the name of the braille table, which you don't want to extend with the HUC8 or with the HUC6 Braille Tables any longer, open that file with Notepad++ and insert a hash ("#") directly in front of the "include huc…".  
   Note: The very last line must remain blank, otherwise the braille output fails.
2. Save the file, restart NVDA and the HUC Braille Tables shouldn't work any longer.

-----------
End of file